// HPCETxnFileGen.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "../../TPC-E/inc/CE.h"
#include "../inc/HPCEInterface_file.h"

using namespace TPCE;

TIdent        iCustomerCount = TPCE::iDefaultCustomerCount;
TIdent        iActiveCustomerCount = iCustomerCount;
UINT          iScaleFactor = 500;                             
UINT          iDaysOfInitialTrades = 300;
UINT          iNbrTxns = 10000;

#ifndef FLAT_IN_PATH
#define FLAT_IN_PATH "flat_in/"
#endif

#ifndef TXN_FILE
#define TXN_FILE "CETXN.DAT"
#endif

#ifndef LOG_FILE
#define LOG_FILE "CETXN.LOG"
#endif

//	Log for messages from TPC-E code
#ifndef EGEN_LOG_FILE
#define EGEN_LOG_FILE "HPCETxnFileGen.LOG"
#endif

char          szInDir[TPCE::iMaxPath];
char          szTxnFile[TPCE::iMaxPath];
char          szLogFile[TPCE::iMaxPath];

/*
 * Program argument handling adapted from TPC-E/EGenLoader.cpp
 */
void Usage()
{
	fprintf( stderr, "Usage:\n" );
	fprintf( stderr, "HPCETxnFileGen [options] \n\n" );
	fprintf( stderr, " Where\n" );
	fprintf( stderr, "  Option                  Default     Description\n" );
	fprintf( stderr, "   -c number              %" PRId64 "        Number of customers in the database\n", iCustomerCount );
	fprintf( stderr, "   -a number              %" PRId64 "        Number of active customers in the \n", iActiveCustomerCount );
	fprintf( stderr, "                                      database (default = -c value)\n" );
	fprintf( stderr, "   -w number              %d         Number of Workdays (8-hour days) of \n", iDaysOfInitialTrades );
	fprintf( stderr, "                                      initial trades in the database\n" );
	fprintf( stderr, "   -f number              %d         Scale factor used for database\n", iScaleFactor );
    fprintf( stderr, "   -i dir                 %-11s Directory for input files\n", FLAT_IN_PATH );
	fprintf( stderr, "   -n number              %d       Number of txns to generate in file\n", iNbrTxns );
	fprintf( stderr, "   -t file                %-10s  Output transaction file spec\n", TXN_FILE );
	fprintf( stderr, "   -l file                %-10s  Output log file spec\n", LOG_FILE );
}

void ParseCommandLine( int argc, char *argv[] )
{
int   arg;
char  *sp;
char  *vp;

  /*
   *  Scan the command line arguments
   */
  for ( arg = 1; arg < argc; ++arg ) {

    /*
     *  Look for a switch
     */
    sp = argv[arg];
    if ( *sp == '-' ) {
      ++sp;
    }
    *sp = (char)tolower( *sp );

    /*
     *  Find the switch's argument.  It is either immediately after the
     *  switch or in the next argv
     */
    vp = sp + 1;
    // Allow for switched that don't have any parameters.
    // Need to check that the next argument is in fact a parameter
    // and not the next switch that starts with '-'.
    //
    if ( (*vp == 0) && ((arg + 1) < argc) && (argv[arg + 1][0] != '-') )
    {
        vp = argv[++arg];
    }

    /*
     *  Parse the switch
     */
    switch ( *sp ) {
      case 'c':
        sscanf(vp, "%"PRId64, &iCustomerCount);
        break;
      case 'a':
        sscanf(vp, "%"PRId64, &iActiveCustomerCount);
        break;
      case 'f':
        iScaleFactor = (UINT) atoi( vp );
        break;
      case 'w':
	    iDaysOfInitialTrades = (UINT) atoi( vp );
        break;
      case 'n':
	    iNbrTxns = (UINT) atoi( vp );
        break;

      case 'i': // Location of input files.
          strncpy(szInDir, vp, sizeof(szInDir));
          if(( '/' != szInDir[ strlen(szInDir) - 1 ] ) && ( '\\' != szInDir[ strlen(szInDir) - 1 ] ))
          {
              strncat( szInDir, "/", sizeof(szInDir) );
          }
          break;

      case 't': 
          strncpy(szTxnFile, vp, sizeof(szTxnFile));
          break;

      case 'l': 
          strncpy(szLogFile, vp, sizeof(szLogFile));
          break;

      default: 
        Usage();
        fprintf( stderr, "Error: Unrecognized option: %s\n",sp);
        exit( ERROR_BAD_OPTION );
    }
  }

}

bool ValidateParameters()
{
    bool bRet = true;

    // Starting customer must be a non-zero integral multiple of load unit size + 1.
    //
    if (iActiveCustomerCount > iCustomerCount)
    {
        cout << "The specified number of active customers (-a " << iActiveCustomerCount
            << ") cannot exceed number of total customers ("
            << iCustomerCount << "). " << endl;

        bRet = false;
    }
	else
	{
		if (iActiveCustomerCount == 0) iActiveCustomerCount = iCustomerCount;
	}

    if (iDaysOfInitialTrades == 0)
    {
        cout << "The specified number of 8-Hour Workdays (-w "
             << (iDaysOfInitialTrades) << ") must be non-zero." << endl;

        bRet = false;
    }

	if (iScaleFactor == 0)
    {
        cout << "The specified scale factor (-f "
             << (iScaleFactor) << ") must be non-zero." << endl;

        bRet = false;
    }

	if (iNbrTxns == 0)
    {
        cout << "The specified number of transactions to generate (-n "
             << (iNbrTxns) << ") must be non-zero." << endl;

        bRet = false;
    }

	return bRet;
}


int _tmain(int argc, char* argv[])
{

	// Establish defaults for directories and files.
    strncpy(szInDir, FLAT_IN_PATH, sizeof(szInDir));
    strncpy(szTxnFile, TXN_FILE, sizeof(szTxnFile));
    strncpy(szLogFile, LOG_FILE, sizeof(szLogFile));

	// Get command line options.
    //
    ParseCommandLine(argc, argv);

    // Validate global parameters that may have been modified on
    // the command line.
    //
    if (!ValidateParameters())
    {
        return ERROR_INVALID_OPTION_VALUE;  // exit from the loader returning a non-zero code
    }

    // Let the user know what settings will be used.
    cout << endl << endl << "Using the following settings." << endl << endl;
    cout << "\tCustomers:\t\t" << iCustomerCount << endl;
    cout << "\tActive Customers:\t" << iActiveCustomerCount << endl;
    cout << "\tScale Factor:\t\t" << iScaleFactor << endl;
    cout << "\tInitial Trade Days:\t" << iDaysOfInitialTrades << endl;
    cout << "\tInput Files Directory:\t" << szInDir << endl;
    cout << "\tNumber of Transactions:\t" << iNbrTxns << endl;
    cout << "\tOutput Txn File: \t" << szTxnFile << endl;
    cout << "\tOutput Txn Log File: \t" << szLogFile << endl;
    cout << endl << endl;

	//	-------------------------------------------------------------------------------
	//	Begin custom creation of file
	//	-------------------------------------------------------------------------------
	try {

//		CInputFiles                     inputFiles;
		DataFileManager                 inputFiles;

		// Create log formatter and logger instance
		CLogFormatTab fmt;
		CEGenLogger logger(eDriverCE, 0, EGEN_LOG_FILE, &fmt);

//		inputFiles.Initialize(eDriverCE, iCustomerCount, iActiveCustomerCount, szInDir);

		PDriverCETxnSettings settings = NULL;

		//	Create the file-producing interface
		CCESUTInterface * pInt = new HPCEInterface_file(szTxnFile, szLogFile);
	
		CCE * pCE = new CCE(pInt, &logger, inputFiles, iCustomerCount, iActiveCustomerCount, 
						iScaleFactor, iDaysOfInitialTrades, 0,settings);
//		pCE->SetClearBufferOption(true);

		UINT i;
		for (i=0; i < iNbrTxns; i++)
		{
			pCE->DoTxn();	//	Each call will write a transaction
			if ((i+1) % 10000 == 0)
			{
				cout << "Wrote " << i+1 << " transactions" << endl;
			}

		}

		delete pCE;
	}
    catch (CBaseErr& err)
    {
        cout << endl << "Error " << err.ErrorNum() << ": " << err.ErrorText();
        if (err.ErrorLoc()) {
            cout << " at " << err.ErrorLoc() << endl;
        } else {
            cout << endl;
        }
        return 1;
    }
    // operator new will throw std::bad_alloc exception if there is no sufficient memory for the request.
    //
    catch (std::bad_alloc&)
    {
        cout << endl << endl << "*** Out of memory ***" << endl;
        return 2;
    }
    catch (std::exception& err)
    {
        cout << endl << endl << "Caught Exception: " << err.what() << endl;
    }
    catch (...)
    {
        cout << endl << endl << "Caught Unknown Exception" << endl;
    }

	cout << endl << "Finished." << endl << endl;
	return 0;
}

