#include "StdAfx.h"

CMSMQSender::CMSMQSender(void)
	: m_hQueue(NULL)
	, m_pMsgProps(NULL)
{
}

CMSMQSender::~CMSMQSender(void)
{
	if (m_pMsgProps)
	{
		if (m_pMsgProps->aPropID)  delete [] m_pMsgProps->aPropID;
		if (m_pMsgProps->aPropVar) delete [] m_pMsgProps->aPropVar;
		if (m_pMsgProps->aStatus)  delete [] m_pMsgProps->aStatus;
		delete [] m_pMsgProps;
	}
}

HRESULT CMSMQSender::Initialize(char * pComputerName, char * pQueueName)
{

	HRESULT hr = MQ_OK;

	CUTF queueName(pQueueName);
	CUTF computerName(pComputerName);
	
	LPWSTR wszFormatName = CMSMQUtil::GetQueueFormatName(queueName.get16(), computerName.get16());		

	//	Validate the input parameter and open the queue.
	if (wszFormatName == NULL)
	{
		return MQ_ERROR_INVALID_PARAMETER;
	}

	//	Open the queue for the process.
	if (m_hQueue == NULL)
	{
		hr = MQOpenQueue(
							wszFormatName, 
							MQ_SEND_ACCESS,
							MQ_DENY_NONE,
							&m_hQueue
							);
		delete [] wszFormatName;
		if (FAILED(hr))
		{
			wprintf(L"MQOpenQueue failed. Error: 0x%X.\n, hr");
			return hr;
		}
	}

	//	Set up message properties for sending.  Values TBD at Send time.
	m_pMsgProps = new MQMSGPROPS;
	if (m_pMsgProps == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;

	m_pMsgProps->aPropID = new MSGPROPID[S_NUMBEROFPROPERTIES];
	if (m_pMsgProps->aPropID == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;
		
	m_pMsgProps->aPropVar = new MQPROPVARIANT[S_NUMBEROFPROPERTIES];
	if (m_pMsgProps->aPropVar == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;
		
	m_pMsgProps->aStatus = new HRESULT[S_NUMBEROFPROPERTIES];
	if (m_pMsgProps->aStatus == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;

	DWORD cPropId = S_ARRAY_PROP_LABEL;
	m_pMsgProps->aPropID[cPropId] = PROPID_M_LABEL;
	m_pMsgProps->aPropVar[cPropId].vt = VT_LPWSTR;
		
	cPropId = S_ARRAY_PROP_BODY;
	m_pMsgProps->aPropID[cPropId] = PROPID_M_BODY;
	m_pMsgProps->aPropVar[cPropId].vt = VT_VECTOR | VT_UI1;

	m_pMsgProps->cProp = S_NUMBEROFPROPERTIES;

	return hr;
}


HRESULT CMSMQSender::Send(void * pMsg, int MsgSize, char * pLabel)
{

	HRESULT hr = MQ_OK;

	//	Set message properties  
	//	Label - convert from ANSI to Wide
	CUTF label(pLabel);
	m_pMsgProps->aPropVar[S_ARRAY_PROP_LABEL].pwszVal = (LPWSTR) label.get16();
	
	//	Body
	m_pMsgProps->aPropVar[S_ARRAY_PROP_BODY].caub.pElems = (UCHAR *) pMsg;
	m_pMsgProps->aPropVar[S_ARRAY_PROP_BODY].caub.cElems = (ULONG) MsgSize;

	//	Send Message
	hr =  MQSendMessage(
                m_hQueue,         // Queue handle
                m_pMsgProps,		    // Pointer to an MQMSGPROPS structu
                MQ_NO_TRANSACTION		// Not in a transaction
                );

	if (FAILED(hr))
	{
		wprintf(L"MQSendMessage failed. Error: 0x%X.\n, hr");
		return hr;
	}
	
	return hr;

}


HRESULT CMSMQSender::Shutdown(void)
{
	HRESULT hr = MQ_OK;

	if (m_hQueue)
	{
		hr = MQCloseQueue(m_hQueue);
		if (FAILED(hr))
			wprintf(L"MQCloseQueue failed. Error: 0x%X. Thread Id: %x\n", hr, GetCurrentThreadId());

		m_hQueue = NULL;
	}

	return hr;
}

