#pragma once
//	From http://ravenspoint.wordpress.com/2010/05/30/world-wide-characters/

/**

Conversion between UTF-8 and UTF-16 strings.

UTF-8 is used by web pages.  It is a variable byte length encoding
of UNICODE characters which is independant of the byte order in a computer word.

UTF-16 is the native Windows UNICODE encoding.

The class stores two copies of the string, one in each encoding,
so should only exist briefly while conversion is done.

This is a wrapper for the WideCharToMultiByte and MultiByteToWideChar
*/
#include <stdlib.h>
#include <Windows.h>

class CUTF
{
	wchar_t * myString16;		///&lt; string in UTF-16
	char * myString8;			///&lt; string in UTF-6

public:
	/// Construct from UTF-16
	CUTF( const wchar_t * ws )
	{
		// store copy of UTF16
		myString16 = (wchar_t * ) malloc( wcslen( ws ) * 2 + 2 );
		wcscpy_s( myString16, wcslen(myString16),  ws );

		// How long will the UTF-8 string be
		int len = WideCharToMultiByte(CP_UTF8, 0, ws, wcslen( ws ),
					NULL, NULL, NULL, NULL );

		// allocate a buffer
		myString8 = (char * ) malloc( len + 1 );

		// convert to UTF-8
		WideCharToMultiByte(CP_UTF8, 0, ws, wcslen( ws ), myString8, len, NULL, NULL);

		// null terminate
		*(myString8+len) = '\0';
	};
	
	///  Construct from UTF8
	CUTF( const char * s )
	{
		myString8 = (char * ) malloc( strlen( s ) + 1 );
		strcpy_s( myString8, strlen(myString8),  s );

		// How long will the UTF-16 string be
		int len = MultiByteToWideChar(CP_UTF8, 0, s, strlen( s ), NULL, NULL );

		// allocate a buffer
		myString16 = (wchar_t * ) malloc( len * 2 + 2 );

		// convert to UTF-16
		MultiByteToWideChar(CP_UTF8, 0, s, strlen( s ), myString16, len);

		// null terminate
		*(myString16+len) = '\0';
	};

	/// get UTF16 version
	const wchar_t * get16() { return myString16; }

	/// get UTF8 version
	const char * get8() { return myString8; }

	/// free buffers
	~CUTF() { free(myString8); free(myString16); }
};
