#ifndef HPCE_TXNFILE_H
#define HPCE_TXNFILE_H

#include <sstream>
#include <fstream>

//#include "../../TPC-E/inc/EGenStandardTypes.h"
#include "../../TPC-E/inc/BaseLogger.h"
#include "../../TPC-E/inc/DriverParamSettings.h"
//#include "../../TPC-E/inc/EGenVersion.h"
//#include "../../TPC-E/inc/locking.h"
//#include "../../TPC-E/inc/DateTime.h"
#include "../../TPC-E/inc/TxnHarnessStructs.h"
#include "HPZREFXPTxnStructs.h"

using namespace TPCE;

namespace HPZREF
{

class CHPCETxnFile 
{
private:
    char        m_Filename[512];
    ofstream    m_OutFile;
    ifstream    m_InFile;
    CMutex      m_OutFileLock;
	unsigned long        m_Recnum;
	bool        m_forcobol;

    char * NullToBlank(char * buf, int size)
	{
		for(int i = 0; i < size; i++)
	    {
			if (buf[i] == '\0') buf[i] = ' ';
		}
     
		return buf;
	}

	char * FormatTS(char * buf, TIMESTAMP_STRUCT structTS, const INT32 dtFormat)
	{
		if (structTS.year == 0 && structTS.hour == 0)
		{
			strcpy(buf, "");
		}
		else
		{
			CDateTime * pDT1 = new CDateTime(&structTS);
			strcpy(buf, pDT1->ToStr(dtFormat));
		}

		return buf;
	}

public:
    CHPCETxnFile(const char *szFilename, bool bWrite=false)
    {
		// TODO: Make settable
		m_forcobol = true;

		m_Recnum = 0;

        // Copy Filename
        strncpy(m_Filename, szFilename, sizeof(m_Filename));

        // Open File
		if (bWrite)
		{
			m_OutFile.open(m_Filename);
			if (!m_OutFile)
			{
				throw CSystemErr(CSystemErr::eCreateFile, "HPCETxnFile::HPCETxnFile");
			}
		}
		else
		{
			m_InFile.open(m_Filename);
			if (!m_InFile)
			{
				throw CSystemErr(CSystemErr::eCreateFile, "HPCETxnFile::HPCETxnFile");
			}
		}

    };

	~CHPCETxnFile()
	{
		if (m_OutFile) m_OutFile.close();
		if (m_InFile) m_InFile.close();
	}

	bool WriteBrokerVolumeTxn(TBrokerVolumeTxnInput * pTxn)
	{
		TXPBrokerVolumeTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		strcpy(fileTxn.sector_name, pTxn->sector_name);	
		for (int idx = 0; idx < F_max_broker_list_len; idx++)
		{
			strcpy(fileTxn.broker_list[idx], pTxn->broker_list[idx]);
		}
		
		return WriteTxn("BV", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteCustomerPositionTxn(TCustomerPositionTxnInput * pTxn)
	{
		TXPCustomerPositionTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.acct_id_idx, TIdent_charfmt, pTxn->acct_id_idx);
		sprintf(fileTxn.cust_id, TIdent_charfmt, pTxn->cust_id);
		sprintf(fileTxn.get_history, bool_charfmt, pTxn->get_history);
		strcpy(fileTxn.tax_id, pTxn->tax_id);
				
		return WriteTxn("CP", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteMarketWatchTxn(TMarketWatchTxnInput * pTxn)
	{
		TXPMarketWatchTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.acct_id, TIdent_charfmt, pTxn->acct_id);
		sprintf(fileTxn.c_id, TIdent_charfmt, pTxn->c_id);
		sprintf(fileTxn.ending_co_id, TIdent_charfmt, pTxn->ending_co_id);
		sprintf(fileTxn.starting_co_id, TIdent_charfmt, pTxn->starting_co_id);
		FormatTS(fileTxn.start_day, pTxn->start_day, dateonly_fmtcode);
		strcpy(fileTxn.industry_name, pTxn->industry_name);
				
		return WriteTxn("MW", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteSecurityDetailTxn(TSecurityDetailTxnInput * pTxn)
	{
		TXPSecurityDetailTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.max_rows_to_return, INT32_charfmt, pTxn->max_rows_to_return);
		sprintf(fileTxn.access_lob_flag, bool_charfmt, pTxn->access_lob_flag);
		FormatTS(fileTxn.start_day, pTxn->start_day, dateonly_fmtcode);
		strcpy(fileTxn.symbol, pTxn->symbol);
				
		return WriteTxn("SD", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteTradeLookupTxn(TTradeLookupTxnInput * pTxn)
	{
		TXPTradeLookupTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		for (int idx = 0; idx < TradeLookupFrame1MaxRows; idx++)
		{
			sprintf(fileTxn.trade_id[idx], TTrade_charfmt, pTxn->trade_id[idx]);
		}
		sprintf(fileTxn.acct_id, TIdent_charfmt, pTxn->acct_id);
		sprintf(fileTxn.max_acct_id, TIdent_charfmt, pTxn->max_acct_id);
		sprintf(fileTxn.frame_to_execute, INT32_charfmt, pTxn->frame_to_execute);
		sprintf(fileTxn.max_trades, INT32_charfmt, pTxn->max_trades);
		FormatTS(fileTxn.end_trade_dts, pTxn->end_trade_dts, datetime_fmtcode);
		FormatTS(fileTxn.start_trade_dts, pTxn->start_trade_dts, datetime_fmtcode);
		strcpy(fileTxn.symbol, pTxn->symbol);
				
		return WriteTxn("TL", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteTradeOrderTxn(TTradeOrderTxnInput * pTxn)
	{
		TXPTradeOrderTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.requested_price, double_charfmt, pTxn->requested_price);
		sprintf(fileTxn.acct_id, TIdent_charfmt, pTxn->acct_id);
		sprintf(fileTxn.is_lifo, INT32_charfmt, pTxn->is_lifo);
		sprintf(fileTxn.roll_it_back, INT32_charfmt, pTxn->roll_it_back);
		sprintf(fileTxn.trade_qty, INT32_charfmt, pTxn->trade_qty);
		sprintf(fileTxn.type_is_margin, INT32_charfmt, pTxn->type_is_margin);
		strcpy(fileTxn.co_name, pTxn->co_name);
		strcpy(fileTxn.exec_f_name, pTxn->exec_f_name);
		strcpy(fileTxn.exec_l_name, pTxn->exec_l_name);
		strcpy(fileTxn.exec_tax_id, pTxn->exec_tax_id);
		strcpy(fileTxn.issue, pTxn->issue);
		strcpy(fileTxn.st_pending_id, pTxn->st_pending_id);
		strcpy(fileTxn.st_submitted_id, pTxn->st_submitted_id);
		strcpy(fileTxn.symbol, pTxn->symbol);
		strcpy(fileTxn.trade_type_id, pTxn->trade_type_id);
				
		return WriteTxn("TO", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteTradeResultTxn(TTradeResultTxnInput * pTxn)
	{
		TXPTradeResultTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.trade_price, double_charfmt, pTxn->trade_price);
		sprintf(fileTxn.trade_id, TTrade_charfmt, pTxn->trade_id);
				
		return WriteTxn("TR", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteMarketFeedTxn(TMarketFeedTxnInput * pTxn)
	{
		TXPMarketFeedTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.unique_symbols, INT32_charfmt, pTxn->unique_symbols);
		strcpy(fileTxn.StatusAndTradeType.status_submitted, pTxn->StatusAndTradeType.status_submitted);
		strcpy(fileTxn.StatusAndTradeType.type_limit_buy, pTxn->StatusAndTradeType.type_limit_buy);
		strcpy(fileTxn.StatusAndTradeType.type_limit_sell, pTxn->StatusAndTradeType.type_limit_sell);
		strcpy(fileTxn.StatusAndTradeType.type_stop_loss, pTxn->StatusAndTradeType.type_stop_loss);
				
		for (int idx = 0; idx < F_max_feed_len; idx++)
		{
			sprintf(fileTxn.Entries[idx].price_quote, double_charfmt, pTxn->Entries[idx].price_quote);
			sprintf(fileTxn.Entries[idx].trade_qty, INT32_charfmt, pTxn->Entries[idx].trade_qty);
			strcpy (fileTxn.Entries[idx].symbol, pTxn->Entries[idx].symbol);
		}

		return WriteTxn("MF", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteTradeStatusTxn(TTradeStatusTxnInput * pTxn)
	{
		TXPTradeStatusTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.acct_id, TIdent_charfmt, pTxn->acct_id);
				
		return WriteTxn("TS", (char *) &fileTxn, sizeof(fileTxn));
	}

	bool WriteTradeUpdateTxn(TTradeUpdateTxnInput * pTxn)
	{
		TXPTradeUpdateTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		for (int idx = 0; idx < TradeUpdateFrame1MaxRows; idx++)
		{
			sprintf(fileTxn.trade_id[idx], TTrade_charfmt, pTxn->trade_id[idx]);
		}

		sprintf(fileTxn.acct_id, TIdent_charfmt, pTxn->acct_id);
		sprintf(fileTxn.max_acct_id, TIdent_charfmt, pTxn->max_acct_id);
		sprintf(fileTxn.frame_to_execute, INT32_charfmt, pTxn->frame_to_execute);
		sprintf(fileTxn.max_trades, INT32_charfmt, pTxn->max_trades);
		sprintf(fileTxn.max_updates, INT32_charfmt, pTxn->max_updates);
		FormatTS(fileTxn.end_trade_dts, pTxn->end_trade_dts, datetime_fmtcode);
		FormatTS(fileTxn.start_trade_dts, pTxn->start_trade_dts, datetime_fmtcode);
		strcpy(fileTxn.symbol, pTxn->symbol);
				
		return WriteTxn("TU", (char *) &fileTxn, sizeof(fileTxn));
	}
	
	bool WriteDataMaintenanceTxn(TDataMaintenanceTxnInput * pTxn)
	{
		TXPDataMaintenanceTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.acct_id, TIdent_charfmt, pTxn->acct_id);
		sprintf(fileTxn.c_id, TIdent_charfmt, pTxn->c_id);
		sprintf(fileTxn.co_id, TIdent_charfmt, pTxn->co_id);
		sprintf(fileTxn.day_of_month, INT32_charfmt, pTxn->day_of_month);
		strcpy(fileTxn.symbol, pTxn->symbol);
		strcpy(fileTxn.table_name, pTxn->table_name);
		strcpy(fileTxn.tx_id, pTxn->tx_id);

		return WriteTxn("DM", (char *) &fileTxn, sizeof(fileTxn));
	}
	
	bool WriteTradeCleanupTxn(TTradeCleanupTxnInput * pTxn)
	{
		TXPTradeCleanupTxnInput fileTxn;
		memset(&fileTxn, 0x20, sizeof(fileTxn));

		sprintf(fileTxn.start_trade_id, TTrade_charfmt, pTxn->start_trade_id);
		strcpy(fileTxn.st_canceled_id, pTxn->st_canceled_id);
		strcpy(fileTxn.st_pending_id, pTxn->st_pending_id);
		strcpy(fileTxn.st_canceled_id, pTxn->st_canceled_id);
		strcpy(fileTxn.st_submitted_id, pTxn->st_submitted_id);
				
		return WriteTxn("TC", (char *) &fileTxn, sizeof(fileTxn));
	}
	
	bool WriteTxn(const char *szPrefix, const char *txn, const long txnSize)
    {
		/* Get the current time and format for timestamp */
		CDateTime dt;	// Automatically sets to current time
		int year, month, day, hour, minute, second, msec;
		dt.GetYMDHMS(&year, &month, &day, &hour,&minute, &second, &msec);		
		
		TFileCommon first;
		memset(&first, 0, sizeof(first));

		strcpy(first.txnID, szPrefix);
		sprintf(first.recNum, "%09u", ++m_Recnum);
		sprintf(first.timeStamp, "%04u%02u%02u%02u%02u%02u%03u", year, month, day, hour, minute, second, msec);

        m_OutFileLock.lock();
		if (m_forcobol)
		{
			NullToBlank((char *) &first, sizeof(first));
			NullToBlank((char *) txn, txnSize);
		}
		m_OutFile.write((const char *) &first, sizeof(first));
		m_OutFile.write((const char *) txn, txnSize);
        m_OutFile << endl;
        m_OutFile.flush();
        if (!m_OutFile)
        {
            throw CSystemErr(CSystemErr::eWriteFile, "HPCETxnFile::WriteTxn");
        }
        m_OutFileLock.unlock();
        return true;

	}

	bool WriteString(string str)
    {
        m_OutFileLock.lock();
		m_OutFile << str;
        m_OutFile << endl;
        m_OutFile.flush();
        if (!m_OutFile)
        {
            throw CSystemErr(CSystemErr::eWriteFile, "HPCETxnFile::WriteString");
        }
        m_OutFileLock.unlock();
        return true;

	}

	bool ReadNext(TFileCERecord * pRecord)
	{
	
		assert(m_InFile.good());

		m_InFile.getline((char *) pRecord, sizeof(TFileCERecord)+1);
		return !(m_InFile.bad() || m_InFile.eof());
	}

};

}   // namespace HPZREF

#endif //HPCE_TXNFILE_H
